from PyQt4.QtCore import *
from PyQt4.QtGui import *
from process import Process
from copy import deepcopy
class ProcessTableModel(QAbstractTableModel):
  """table model for process table"""
  def __init__(self, parent=0):
    super(QAbstractTableModel, self).__init__()
    self.background=[QBrush(Qt.white), QBrush(Qt.lightGray)]
    self.null = QVariant()
    self.__items = []
    self.__columns = ["Name", "Start", "Duration", "Progress"]
    self.__scheduled = False
    
  def columnCount(self, parent=QModelIndex()):
    return 4
    
    
  def data(self, index, role):
    if not index.isValid():
      return self.null

    if index.row() >= self.rowCount:
      return self.null
      
    if role == Qt.DisplayRole or role == Qt.EditRole:
      if (not self.__scheduled) or (index.column() != 1):
	return self.fieldForColumnRow(index.column(), index.row())
      return "C: %d R: %d" % (self.__items[index.row()].realStart(), self.__items[index.row()].start())
      
    elif role == Qt.BackgroundRole:
      return self.background[index.row()%2]
    elif role == Qt.TextAlignmentRole:
      if ( 0 < index.column() <= 2):
	return Qt.AlignHCenter + Qt.AlignVCenter
    else:
      return self.null


  def flags(self, index):
    if index.isValid():
      if (index.column() != 3) and (not self.__scheduled):
        return Qt.ItemIsEditable | Qt.ItemIsEnabled
      return Qt.ItemIsEnabled
    return Qt.NoItemFlags


  def rowCount(self, parent=QModelIndex()):
    return len (self.__items)


  def setData(self, index, value, role=Qt.EditRole):
    if index.isValid() and role == Qt.EditRole:
      self.fieldForColumnRow(index.column(), index.row(), value)
      self.emit(SIGNAL("dataChanged(index, index)"))
      return True
    return False


  def index(self, row, column, parent=QModelIndex()):
    if not self.hasIndex(row, column, parent):
      return QModelIndex()
    return self.createIndex(row, column)
    
    
  def insertRow(self, row, parent=QModelIndex()):
    self.beginInsertRows(parent, self.rowCount()-1, self.rowCount()-1)
    self.__items.extend([Process("Process {0}".format(len(self.__items)+1), 0, 0)])
    self.endInsertRows()
    return True


  def items(self):
      return self.__items


  def fieldForColumnRow(self, column, row, value=None):
      func = [self.__items[row].name, self.__items[row].start, self.__items[row].duration, self.__items[row].progress][column]
      return apply(func, [value])


  def headerData(self, section, orientation, role=Qt.DisplayRole):
    if role == Qt.DisplayRole:
      if orientation == Qt.Horizontal:
	return self.__columns[section]
      elif orientation == Qt.Vertical:
	return section+1


  def unschedule(self):
    self.__scheduled = False
    for p in self.__items:
      p.progress(0)
      p.start(p.realStart())
    start = self.index(0, 1)
    end = self.index (self.rowCount()-1, self.columnCount()-1)
    self.dataChanged.emit (start, end)


  def schedule(self, items):
    self.__items = deepcopy (items)
    self.__scheduled = True
    start = self.index(0, 1)
    end = self.index (self.rowCount()-1, self.columnCount()-1)
    self.dataChanged.emit (start, end)

    
class ProcessTableDelegate(QItemDelegate):
  """delegate QTableWidget for showing process table"""
  
  def createEditor(self, parent, option, index):
    if index.column() not in (0,3):
      editor = QSpinBox(parent)
      editor.setMinimum(0)
      editor.setMaximum(100)
      return editor
    else:
      return QLineEdit(parent)


  def setEditorData(self, editor, index):
    if index.column() != 0:
      value = index.model().data(index, Qt.EditRole)
      editor.setValue(value)


  def setModelData(self, editor, model, index):
    if index.column() != 0:
      editor.interpretText()
      value = editor.value()
      model.setData(index, value)
    else:
      value = editor.text().trimmed()
      if value.isEmpty():
        model.setData(index, "Process {0}".format(index.row()+1))
      else:
        model.setData(index, value)


  def paint(self, painter, option, index):
    if (index.column() == 3):
      progress = index.data().toInt()[0]
      progressBarOption = QStyleOptionProgressBar()
      progressBarOption.rect = option.rect
      progressBarOption.minimum = 0
      progressBarOption.maximum = 100
      progressBarOption.progress = progress
      progressBarOption.text = QString.number(progress) + "%"
      progressBarOption.textVisible = True
      QApplication.style().drawControl(QStyle.CE_ProgressBar, progressBarOption, painter)
    else:
      QStyledItemDelegate().paint(painter, option, index)